;; Generated package description from ivy-avy.el  -*- no-byte-compile: t -*-
(define-package "ivy-avy" "0.14.0.0.20230410.182616" "Avy integration for Ivy" '((emacs "24.5") (ivy "0.14.0") (avy "0.5.0")) :commit "522c2ed7a62d3c7ff9e7ea455ad3c5d63b3c4b54" :authors '(("Oleh Krehel" . "ohwoeowho@gmail.com")) :maintainer '("Oleh Krehel" . "ohwoeowho@gmail.com") :keywords '("convenience") :url "https://github.com/abo-abo/swiper")
